@@ -1,7 +1,7 @@
 # pylint: disable=C0111,R0902,R0913
 # Smartsheet Python SDK.
 #
-# Copyright 2016 Smartsheet.com, Inc.
+# Copyright 2018 Smartsheet.com, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"): you may
 # not use this file except in compliance with the License. You may obtain
@@ -56,9 +56,6 @@ class Sheets(object):
         _op['method'] = 'POST'
         _op['path'] = '/sheets/' + str(sheet_id) + '/columns'
         _op['json'] = list_of_columns
-        # filter before we go
-        for item in _op['json']:
-            item.pre_request_filter = 'add_columns'
 
         expected = ['Result', 'Column']
 
@@ -118,9 +115,6 @@ class Sheets(object):
         _op['method'] = 'POST'
         _op['path'] = '/sheets/' + str(sheet_id) + '/rows'
         _op['json'] = list_of_rows
-        # filter before we go
-        for item in _op['json']:
-            item.pre_request_filter = 'add_rows'
 
         expected = ['Result', 'Row']
 
@@ -182,10 +176,6 @@ class Sheets(object):
         _op['json'] = list_of_rows
         _op['query_params']['allowPartialSuccess'] = 'true'
 
-        # filter before we go
-        for item in _op['json']:
-            item.pre_request_filter = 'add_rows'
-
         expected = ['BulkItemResult', 'Row']
 
         prepped_request = self._base.prepare_request(_op)
@@ -237,10 +227,10 @@ class Sheets(object):
             container_destination_obj
                 (ContainerDestination): Container Destination object.
             include (list[str]): A comma-separated list of
-                optional elements to include in the response. Valid list
-                values: attachments, cellLinks, data, discussions, forms,
-                    ruleRecipients, rules, shares, all (deprecated).
-            omit (list[str]): A comma-seperated list of optional elements
+                optional elements to include in the response. Valid list values:
+                attachments, cellLinks, data, discussions, forms, ruleRecipients,
+                rules, shares, all (deprecated).
+            omit (list[str]): A comma-separated list of optional elements
                 to omit. Only current valid value is sheetHyperlinks
 
         Returns:
@@ -304,7 +294,7 @@ class Sheets(object):
         _op['query_params']['ids'] = ids
         _op['query_params']['ignoreRowsNotFound'] = ignore_rows_not_found
 
-        expected = 'Result'
+        expected = ['Result', 'NumberObjectValue']
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -324,7 +314,7 @@ class Sheets(object):
         _op['method'] = 'DELETE'
         _op['path'] = '/sheets/' + str(sheet_id) + '/shares/' + str(share_id)
 
-        expected = 'Result'
+        expected = ['Result', None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -343,7 +333,7 @@ class Sheets(object):
         _op['method'] = 'DELETE'
         _op['path'] = '/sheets/' + str(sheet_id)
 
-        expected = 'Result'
+        expected = ['Result', None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -372,17 +362,16 @@ class Sheets(object):
 
         return response
 
-    def get_columns(self, sheet_id, include=None, page_size=100, page=1,
-                    include_all=False):
+    def get_columns(self, sheet_id, include=None, page_size=None, page=None,
+                    include_all=None):
         """Get all columns belonging to the specified Sheet.
 
         Args:
             sheet_id (int): Sheet ID
             include (str): (future)
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
 
@@ -476,7 +465,7 @@ class Sheets(object):
         return response
 
     def get_sheet(self, sheet_id, include=None, exclude=None, row_ids=None,
-                  row_numbers=None, column_ids=None, page_size=100, page=1):
+                  row_numbers=None, column_ids=None, page_size=None, page=None, if_version_after=None):
         """Get the specified Sheet.
 
         Get the specified Sheet. Returns the Sheet, including Rows,
@@ -487,7 +476,7 @@ class Sheets(object):
             sheet_id (int): Sheet ID
             include (list[str]): A comma-separated list of
                 optional elements to include in the response. Valid list
-                values: attachments, discussions, format, filters,
+                values: attachments, crossSheetReferences, discussions, format, filters,
                 filterDefinitions, ownerInfo, source, rowWriterInfo.
             exclude (str): Response will not include cells
                 that have never contained any data.
@@ -501,9 +490,10 @@ class Sheets(object):
                 columns in the 'columns' array, and individual rows' 'cells'
                 array will only contain cells in the specified columns.
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
+            if_version_after (int): only fetch Sheet if more recent version
+                available.
 
         Returns:
             Sheet
@@ -518,6 +508,7 @@ class Sheets(object):
         _op['query_params']['columnIds'] = column_ids
         _op['query_params']['pageSize'] = page_size
         _op['query_params']['page'] = page
+        _op['query_params']['ifVersionAfter'] = if_version_after
 
         expected = 'Sheet'
         prepped_request = self._base.prepare_request(_op)
@@ -663,19 +654,19 @@ class Sheets(object):
 
         return response
 
-    def list_shares(self, sheet_id, page_size=100, page=1,
-                    include_all=False, include_workspace_shares=False):
+    def list_shares(self, sheet_id, page_size=None, page=None,
+                    include_all=None, include_workspace_shares=False):
         """Get the list of all Users and Groups to whom the specified Sheet is
         shared, and their access level.
 
         Args:
             sheet_id (int): Sheet ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
+            include_workspace_shares(bool): Include Workspace shares
 
         Returns:
             IndexResult
@@ -696,8 +687,8 @@ class Sheets(object):
 
         return response
 
-    def list_sheets(self, include=None, page_size=100, page=1,
-                    include_all=False, modified_since=None):
+    def list_sheets(self, include=None, page_size=None, page=None,
+                    include_all=None, modified_since=None):
         """Get the list of all Sheets the User has access to, in alphabetical
         order, by name.
 
@@ -706,11 +697,11 @@ class Sheets(object):
                 optional elements to include in the response. Valid list
                 values: ownerInfo, sheetVersion, source.
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
+            modified_since(datetime): Return sheets modified since provided datetime
 
         Returns:
             IndexResult
@@ -790,8 +781,6 @@ class Sheets(object):
         _op['method'] = 'POST'
         _op['path'] = '/sheets/' + str(sheet_id) + '/move'
         _op['json'] = container_destination_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'move_sheet'
 
         expected = ['Result', 'Sheet']
 
@@ -838,7 +827,7 @@ class Sheets(object):
         _op['path'] = '/sheets/' + str(sheet_id) + '/rows/emails'
         _op['json'] = multi_row_email_obj
 
-        expected = 'Result'
+        expected = ['Result', None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -860,7 +849,7 @@ class Sheets(object):
         _op['path'] = '/sheets/' + str(sheet_id) + '/emails'
         _op['json'] = sheet_email_obj
 
-        expected = 'Result'
+        expected = ['Result', None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -925,8 +914,6 @@ class Sheets(object):
         _op['method'] = 'PUT'
         _op['path'] = '/sheets/' + str(sheet_id) + '/publish'
         _op['json'] = sheet_publish_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'set_publish_status'
 
         expected = ['Result', 'SheetPublish']
 
@@ -956,8 +943,6 @@ class Sheets(object):
         _op['path'] = '/sheets/' + str(sheet_id) + '/shares'
         _op['query_params']['sendEmail'] = send_email
         _op['json'] = share_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'share_sheet'
 
         expected = ['Result', 'Share']
 
@@ -991,8 +976,6 @@ class Sheets(object):
         _op['path'] = '/sheets/' + str(sheet_id) + '/columns/' + str(
             column_id)
         _op['json'] = column_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'update_column'
 
         expected = ['Result', 'Column']
 
@@ -1027,9 +1010,6 @@ class Sheets(object):
         _op['method'] = 'PUT'
         _op['path'] = '/sheets/' + str(sheet_id) + '/rows'
         _op['json'] = list_of_rows
-        # filter before we go
-        for item in _op['json']:
-            item.pre_request_filter = 'update_rows'
 
         expected = ['Result', 'Row']
 
@@ -1066,10 +1046,6 @@ class Sheets(object):
         _op['json'] = list_of_rows
         _op['query_params']['allowPartialSuccess'] = 'true'
 
-        # filter before we go
-        for item in _op['json']:
-            item.pre_request_filter = 'update_rows'
-
         expected = ['BulkItemResult', 'Row']
 
         prepped_request = self._base.prepare_request(_op)
@@ -1098,8 +1074,6 @@ class Sheets(object):
         _op['method'] = 'PUT'
         _op['path'] = '/sheets/' + str(sheet_id) + '/shares/' + str(share_id)
         _op['json'] = share_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'update_share'
 
         expected = ['Result', 'Share']
 
@@ -1122,8 +1096,6 @@ class Sheets(object):
         _op['method'] = 'PUT'
         _op['path'] = '/sheets/' + str(sheet_id)
         _op['json'] = sheet_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'update_sheet'
 
         expected = ['Result', 'Sheet']
 
@@ -1132,17 +1104,16 @@ class Sheets(object):
 
         return response
 
-    def list_update_requests(self, sheet_id, page_size=100, page=1,
-                        include_all=False):
+    def list_update_requests(self, sheet_id, page_size=None, page=None,
+                             include_all=None):
         """Get the list of all Sheet UpdateRequests.
 
         Args:
             sheet_id (int): Sheet ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
-            include_all (bool): If true, include all results
+                return per page.
+            page (int): Which page to return.
+            include_all(bool): If true, include all results
                 (i.e. do not paginate).
 
         Returns:
@@ -1196,8 +1167,6 @@ class Sheets(object):
         _op['method'] = 'POST'
         _op['path'] = '/sheets/' + str(sheet_id) + '/updaterequests'
         _op['json'] = update_request_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'create_update_request'
 
         expected = ['Result', 'UpdateRequest']
 
@@ -1220,17 +1189,18 @@ class Sheets(object):
         _op['method'] = 'DELETE'
         _op['path'] = '/sheets/' + str(sheet_id) + '/updaterequests/' + str(update_request_id)
 
-        expected = 'Result'
+        expected = ['Result', None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
         return response
 
-    def update_update_request(self, sheet_id, update_request_obj):
+    def update_update_request(self, sheet_id, update_request_id, update_request_obj):
         """Updates an UpdateRequest for the specified Rows(s) within the Sheet.
 
         Args:
             sheet_id (int): Sheet ID
+            update_request_id: Update request ID
             update_request_obj (UpdateRequest): UpdateRequest object
 
         Returns:
@@ -1238,10 +1208,8 @@ class Sheets(object):
         """
         _op = fresh_operation('create_update_request')
         _op['method'] = 'PUT'
-        _op['path'] = '/sheets/' + str(sheet_id) + '/updaterequests/' + str(update_request_obj.id)
+        _op['path'] = '/sheets/' + str(sheet_id) + '/updaterequests/' + str(update_request_id)
         _op['json'] = update_request_obj
-        # filter before we go
-        _op['json'].pre_request_filter = 'update_update_request'
 
         expected = ['Result', 'UpdateRequest']
 
@@ -1250,16 +1218,15 @@ class Sheets(object):
 
         return response
 
-    def list_sent_update_requests(self, sheet_id, page_size=100, page=1,
-                        include_all=False):
+    def list_sent_update_requests(self, sheet_id, page_size=None, page=None,
+                                  include_all=None):
         """Get the list of all Sent UpdateRequests.
 
         Args:
             sheet_id (int): Sheet ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
 
@@ -1314,22 +1281,21 @@ class Sheets(object):
         _op['method'] = 'DELETE'
         _op['path'] = '/sheets/' + str(sheet_id) + '/sentupdaterequests/' + str(sent_update_request_id)
 
-        expected = 'Result'
+        expected = ['Result', None]
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
         return response
 
-    def list_filters(self, sheet_id, page_size=100, page=1,
-                        include_all=False):
+    def list_filters(self, sheet_id, page_size=None, page=None,
+                     include_all=None):
         """Returns a list of all saved sheet filters
 
         Args:
             sheet_id (int): Sheet ID
             page_size (int): The maximum number of items to
-                return per page. Defaults to 100.
-            page (int): Which page to return. Defaults to 1
-                if not specified.
+                return per page.
+            page (int): Which page to return.
             include_all (bool): If true, include all results
                 (i.e. do not paginate).
 
@@ -1384,7 +1350,192 @@ class Sheets(object):
         _op['method'] = 'DELETE'
         _op['path'] = '/sheets/' + str(sheet_id) + '/filters/' + str(filter_id)
 
-        expected = 'Result'
+        expected = ['Result', None]
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def list_cross_sheet_references(self, sheet_id, page_size=None, page=None,
+                                  include_all=None):
+        """Get the list of all CrossSheetReferences for this Sheet.
+
+        Args:
+            sheet_id (int): Sheet ID
+            page_size (int): The maximum number of items to
+                return per page.
+            page (int): Which page to return.
+            include_all (bool): If true, include all results
+                (i.e. do not paginate).
+
+        Returns:
+            IndexResult
+        """
+        _op = fresh_operation('list_cross_sheet_references')
+        _op['method'] = 'GET'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/crosssheetreferences'
+        _op['query_params']['pageSize'] = page_size
+        _op['query_params']['page'] = page
+        _op['query_params']['includeAll'] = include_all
+
+        expected = ['IndexResult', 'CrossSheetReference']
+
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def get_cross_sheet_reference(self, sheet_id, cross_sheet_reference_id):
+        """Get the CrossSheetReference.
+
+        Args:
+            sheet_id (int): Sheet ID
+            cross_sheet_reference_id (int): CrossSheetReferenceID
+
+        Returns:
+            CrossSheetReference
+        """
+        _op = fresh_operation('get_cross_sheet_reference')
+        _op['method'] = 'GET'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/crosssheetreferences/' + str(cross_sheet_reference_id)
+
+        expected = 'CrossSheetReference'
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def create_cross_sheet_reference(self, sheet_id, cross_sheet_reference_obj):
+        """Creates a CrossSheetReference for the specified Sheet.
+
+        Args:
+            sheet_id (int): Sheet ID
+            cross_sheet_reference_obj (CrossSheetReference): CrossSheetReference object
+
+        Returns:
+            Result
+        """
+        _op = fresh_operation('create_cross_sheet_reference')
+        _op['method'] = 'POST'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/crosssheetreferences'
+        _op['json'] = cross_sheet_reference_obj
+
+        expected = ['Result', 'CrossSheetReference']
+
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def list_automation_rules(self, sheet_id, page_size=None, page=None,
+                                  include_all=None):
+        """Get the list of all AutomationRules for this Sheet.
+
+        Args:
+            sheet_id (int): Sheet ID
+            page_size (int): The maximum number of items to
+                return per page.
+            page (int): Which page to return.
+            include_all (bool): If true, include all results
+                (i.e. do not paginate).
+
+        Returns:
+            IndexResult
+        """
+        _op = fresh_operation('list_automation_rules')
+        _op['method'] = 'GET'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/automationrules'
+        _op['query_params']['pageSize'] = page_size
+        _op['query_params']['page'] = page
+        _op['query_params']['includeAll'] = include_all
+
+        expected = ['IndexResult', 'AutomationRule']
+
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def get_automation_rule(self, sheet_id, automation_rule_id):
+        """Get the AutomationRule.
+
+        Args:
+            sheet_id (int): Sheet ID
+            automation_rule_id (long): AutomationRuleID
+
+        Returns:
+            AutomationRule
+        """
+        _op = fresh_operation('get_automation_rule')
+        _op['method'] = 'GET'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/automationrules/' + str(automation_rule_id)
+
+        expected = 'AutomationRule'
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def update_automation_rule(self, sheet_id, automation_rule_id, automation_rule_obj):
+        """Updates an AutomationRule for the specified Sheet.
+
+        Args:
+            sheet_id (int): Sheet ID
+            automation_rule_id: AutomationRule ID
+            automation_rule_obj (AutomationRule): AutomationRule object
+
+        Returns:
+            Result
+        """
+        _op = fresh_operation('update_automation_rule')
+        _op['method'] = 'PUT'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/automationrules/' + str(automation_rule_id)
+        _op['json'] = automation_rule_obj
+
+        expected = ['Result', 'AutomationRule']
+
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def delete_automation_rule(self, sheet_id, automation_rule_id):
+        """Deletes an AutomationRule for the specified Sheet.
+
+        Args:
+            sheet_id (int): Sheet ID
+            automation_rule_id (int): AutomationRule ID
+
+        Returns:
+            Result
+        """
+        _op = fresh_operation('delete_automation_rule')
+        _op['method'] = 'DELETE'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/automationrules/' + str(automation_rule_id)
+
+        expected = ['Result', None]
+        prepped_request = self._base.prepare_request(_op)
+        response = self._base.request(prepped_request, expected, _op)
+
+        return response
+
+    def sort_sheet(self, sheet_id, sort_specifier_obj):
+        """Sort Sheet according to SortSpecifier.
+
+        Args:
+            sheet_id (int): Sheet ID
+            sort_specifier_obj (SortSpecifier): SortSpecifier object
+
+        Returns:
+            Sheet
+        """
+        _op = fresh_operation('sort_sheet')
+        _op['method'] = 'POST'
+        _op['path'] = '/sheets/' + str(sheet_id) + '/sort'
+        _op['json'] = sort_specifier_obj
+
+        expected = 'Sheet'
+
         prepped_request = self._base.prepare_request(_op)
         response = self._base.request(prepped_request, expected, _op)
 
@@ -1396,6 +1547,8 @@ class Sheets(object):
         Note: returns the first matching title found.
 
         Args:
+            sheet_id(int): Sheet ID
+            title(str): Title search string
             include (str): (future).
         """
         all_columns = self.get_columns(sheet_id, include_all=True)
@@ -1406,7 +1559,7 @@ class Sheets(object):
 
     def get_sheet_by_name(self, name, include=None, exclude=None,
                           row_ids=None, row_numbers=None, column_ids=None,
-                          page_size=100, page=1):
+                          page_size=None, page=None):
         """For those times when you don't know the Sheet Id.
 
         Note: returns the first matching name found.
